/* $Workfile: OtxDrv.h $   $Revision: 57 $    $Modtime: 1/30/02 11:33a $  */
#ifndef OTX_DRV_H
#define OTX_DRV_H
/******************************************************************************

  Copyright (C) 1998, 1999 Odin TeleSystems Inc., All Rights Reserved

  @doc PublicHwApi OtxDrv

  OTX Generic Hardware Driver API

  @module
  OtxDrv | This file contains prototypes for the Driver Functions

  @contents2
  OtxDrv.h |

  @index func | OtxDrv

  @contents2
  OTX Driver Level Functions - OtxDrvXXX() | Driver level functions

  @head3 OtxDrvXXX() Function Reference |

  @index func | OtxDrv

******************************************************************************/


#ifdef __cplusplus
extern "C" {
#endif

#include "OtxDrvD.h"    // OTX Driver type definitions
#include "OtxDef.h"     // OTX specific macro definitions etc.
#include "OtxDev.h"     // OTX specific device type definitions
#include "OtxType.h"    // OTX specific type definitions

/******************************************************************************

  @func
    Initialize the driver API library.

  @comm
    Sets up a communication channel with the driver.

  @xref
    <f OtxDrvReleaseLib>

******************************************************************************/
OTX_RESULT OtxDrvConnectLib(
  void
);


/******************************************************************************

  @func
    Releases all references that this driver is holding.

  @comm
    All handles that an application is holding are closed automatically when a
    process terminates. If the application is about to terminate, calling
    this function is not necessary. However a server application may need
    this function if it never terminates.

  @xref
    <f OtxDrvConnectLib>

******************************************************************************/
OTX_RESULT OtxDrvReleaseLib(
  void
);



/******************************************************************************

  @func
    Gets the current time from the host.

  @xref
    <t OTX_DATETIME>

******************************************************************************/
OTX_RESULT OtxDrvGetCurrentTime(
  OUT OTX_DATETIME  *pnTimeStamp    // @parm Current time of the host
);



// @todo Future functionality
/******************************************************************************

  @func
    Get the an attribute of a device

  @comm
    This function can, for example, be used to retrieve the revision of the
    currently loaded driver, or a firmware revision of a physical device.
    Different devices have different attributes, see the header file for the
    device in question for specific information

  @xref
    <f OtxDrvGetAttributeValue>
    <nl><f OtxDrvSetAttributeValue>
    <nl><f OtxDrvResultCode2String>
    <nl><f OtxDrvEventCode2String>
    <nl><f OtxDrvConnectLib>

******************************************************************************/
OTX_RESULT OtxDrvGetAttributeType(
  IN OTX_HANDLE       hDevice,          // @parm Handle to the device being queried
  IN OTX_ATTRIB_ID    nAttribId,        // @parm ID of attribute
  OUT OtxAttribTypeE  *peAttribType,    // @parm Type of the attribute, if attribute exists
  OUT OTX_UINT32      *pcbAttribValue,  // @parm Size in bytes of attribute value (Get value with <f OtxDrvGetAttributeValue>)
  OUT OTX_BOOL        *pbReadOnly       // @parm Is attribute writable or read-only

);


/******************************************************************************

  @func
    Get an attribute of a device

  @xref
    <f OtxDrvGetAttributeValue>

******************************************************************************/
OTX_RESULT OtxDrvGetAttributeValueBool(
  IN OTX_HANDLE       hDevice,
  IN OTX_ATTRIB_ID    nAttribId,
  OUT OTX_BOOL        *pAttribValue
);


/******************************************************************************

  @func
    Get an attribute of a device

  @xref
    <f OtxDrvGetAttributeValue>

******************************************************************************/
OTX_RESULT OtxDrvGetAttributeValueUint32(
  IN OTX_HANDLE       hDevice,
  IN OTX_ATTRIB_ID    nAttribId,
  OUT OTX_UINT32      *pAttribValue
);


/******************************************************************************

  @func
    Get an attribute of a device

  @xref
    <f OtxDrvGetAttributeValue>

******************************************************************************/
OTX_RESULT OtxDrvGetAttributeValueSz(
  IN OTX_HANDLE       hDevice,
  IN OTX_ATTRIB_ID    nAttribId,
  OUT char            *pAttribValueBuf,
  IN OTX_INT32        cbAttribValueBuf
);

/******************************************************************************

  @func
    Get an attribute of a device

  @comm
    This function can, for example be used to retrieve the revision of the
    currently loaded driver, or a firmware revision of a physical device.

  @xref
    <f OtxDrvGetAttributeType>
    <nl><f OtxDrvSetAttributeValue>
    <nl><f OtxDrvResultCode2String>
    <nl><f OtxDrvEventCode2String>
    <nl><f OtxDrvConnectLib>

******************************************************************************/
OTX_RESULT OtxDrvGetAttributeValue(
  IN OTX_HANDLE       hDevice,            // @parm Handle to the device being queried
  IN OTX_ATTRIB_ID    nAttribId,          // @parm ID of attribute
  OUT OTX_PVOID       pAttribValueBuf,    // @parm Buffer to hold the value of the attribute
  IN OTX_UINT32       cbAttribValueBuf,   // @parm Size of the attribValueBuf in bytes
  OUT OTX_UINT32      *pcbAttribValue     // @parm Bytes actually used in attribValueBuf
);


/******************************************************************************

  @func
    Set an attribute of a device

  @xref
    <f OtxDrvGetAttributeValue>

******************************************************************************/
OTX_RESULT OtxDrvSetAttributeValueBool(
  IN  OTX_HANDLE      hDevice,
  IN  OTX_ATTRIB_ID   nAttribId,
  IN  OTX_BOOL        attribValue
);


/******************************************************************************

  @func
    Get an attribute of a device

  @xref
    <f OtxDrvGetAttributeValue>

******************************************************************************/
OTX_RESULT OtxDrvSetAttributeValueUint32(
  IN  OTX_HANDLE      hDevice,
  IN  OTX_ATTRIB_ID   nAttribId,
  IN  OTX_UINT32      attribValue
);


/******************************************************************************

  @func
    Get an attribute of a device

  @xref
    <f OtxDrvGetAttributeValue>

******************************************************************************/
OTX_RESULT OtxDrvSetAttributeValueSz(
  IN  OTX_HANDLE      hDevice,
  IN  OTX_ATTRIB_ID   nAttribId,
  IN  char            *pAttribValue
);


/******************************************************************************

  @func
    Set the an attribute of a device

  @comm
    This function can, for example be used to disable and enable certain
    functions of a device, for example enable or disable reception
    of Caller ID for a Pots device . Note that some attributes
    are read/only. If an attempt to set a read only attribute is made
    an error code is returned as result.

  @xref
    <f OtxDrvGetAttributeType>
    <nl><f OtxDrvGetAttributeValue>
    <nl><f OtxDrvResultCode2String>
    <nl><f OtxDrvEventCode2String>
    <nl><f OtxDrvConnectLib>

******************************************************************************/
OTX_RESULT OtxDrvSetAttributeValue(
  IN OTX_HANDLE       hDevice,            // @parm Handle to the device being queried
  IN OTX_ATTRIB_ID    nAttribId,          // @parm ID of attribute
  OUT OTX_UINT8       attribValueBuf[],   // @parm Value of the attribute, if attribute exists
  IN OTX_UINT32       cbAttribValueBuf    // @parm Size in bytes of the buffer for the value
);


/******************************************************************************

  @func
    Gets a handle to an alredy existing Device Object

  @comm

    The returned handle can only be used for query operations such as
    getting attributes and for opening child devices which requires the
    handle of the parent.

  @xref
    <f OtxDrvOpenPhysicalDevice>
    <nl><t OtxPhysicalDeviceTypeE>
    <nl><t OtxLogicalDeviceTypeE>
    <nl><t OTX_HANDLE>

******************************************************************************/
OTX_RESULT OtxDrvGetDeviceHandle(
  IN  OTX_HANDLE    hDevice,
  IN  OtxDeviceType nDeviceType,
  IN  OTX_INT32     nDevice,
  OUT OTX_HANDLE    *phDevice
);


/******************************************************************************

  @func
    Opens a Device Object representing a physical hardware device, which can
    be an entire board or an integrated circuit (chip) on a board. The number of
    physical devices is set in hardware and cannot change dynamically.

  @comm
    Each physical devices in the system are identified by 3 parameters
    - Parent          : Physical Device contaning the device
    - Type            : Type of the Physical Device
    - Sequence Number : Number of the device within the host and with its type

    The physical devices can be classified in 2 categories:

  - BOARD Devices: A board device represents one OTX Network Interface card or
    a Daughter Board Module. The board device is the placeholder and controller
    of other physical devices on the board.

  - CHIP Devices: A chip device represents a physical integrated circuit
    on the board performing one or several functions. A logical device can
    be created to represent each of the functions implemented by a chip device.
    For example, a Digital Signal Processor is a physical "chip" device.
    However, the DSP software may implement several functions which then
    can be represented by a multiple of logical devices.

    The chip devices can be further categorized as follows:

  - Line Interface Devices: A hardware device that interfaces with the external
    network and converts/maps the non-TTL level external data interface to the
    internal serial data highways. Typically an actual line interface transceiver
    chip on a board. Examples of Line Interface Devices include T1/E1, BRI, POTS,
    T3/E3 Lis.

  - Processor Devices: A processor (general purpose micro processor or Digital
    Signal Processor) is an integrated circuit that implements certain functionality
    (logical devices) under software control.

  - Switch Device: A device connecting and disconnecting data-paths.

  - Other miscellaneous chips.

  @xref
    <f OtxDrvClose>
    <nl><t OtxPhysicalDeviceTypeE>
    <nl><t OtxLogicalDeviceTypeE>
    <nl><t OTX_HANDLE>

******************************************************************************/
OTX_RESULT OtxDrvOpenPhysicalDevice(
  IN  OTX_HANDLE hHostDevice,          // @parm Parent (typically Board Device) who owns the device we are connecting to
                                       //       Specify NULL when opening a board device
  IN  OtxPhysicalDeviceTypeE eDevType, // @parm Class of device we are looking for. See <t OtxPhysicalDeviceTypeE>
  IN  OTX_INT32  nDevice,              // @parm Sequence number of device within it's class
  IN  OTX_INT32  nEventSourceId,       // @parm User specified ID to be used by this device as the Source ID for the events generated by the device
  IN  OTX_HANDLE hEventQueue,          // @parm Handle to separate event queue (NULL creates device specific event queue)
  OUT OTX_HANDLE *phPhysDevice         // @parm Handle to the new physical device (if result code OTX_S_OK)
);



/******************************************************************************

  @func
    Opens a Device Object representing a physical hardware device, for which 
    a handle has already been acquired (e.g with <f OtxDrvEnumDevice>). Otherwise
    this function has the same functionality as <f OtxDrvOpenPhysicalDevice>.

  @xref
    <f OtxDrvEnumDevice>
    <f OtxDrvOpenPhysicalDevice>

******************************************************************************/
OTX_RESULT OtxDrvOpenPhysicalDeviceEx(
  IN  OTX_HANDLE hPhysDevice,          // @parm Handle to the physical device
  IN  OTX_INT32  nEventSourceId,       // @parm User specified ID to be used by this device as the Source ID for the events generated by the device
  IN  OTX_HANDLE hEventQueue           // @parm Handle to separate event queue (NULL creates device specific event queue)
);



/******************************************************************************

  @func
    Create a Device Object to represent a logical device which is implemented
    with hardware, firmware, or software within a physical device.

  @comm
    The logical devices are differenciated from the phyiscal devices by the fact
    the number of logical devices is not pre-determined. More logical devices can
    be requested by the user and allocated by the system if needed. Examples of
    logical devices include HDLC Receiver, HDLC Sender, Tone Detector, Tone Generator,
    etc.

    The following Logical Devices are supported by the OTX Driver:

    - Tone Devices  : Logical devices for tone generation and reception (See OtxTone.h)
    - Data Devices  : Devices for sending, receiving, and manipulation of
                      Raw data (See OtxData.h)
    - HDLC Devices  : Logical devices implementing HDLC Sender and Receiver
                      Devices (See OtxHdlc.h)
    - CAS Devices   : Devices implementin Channel Associated Signalling (CAS)
                      functions (See OtxCas1.h)
    - Modem Devices : Devices implementing Modem emulation
    - Fax Devices   : Devices implementin Fax emulation
    - Voice Devices : Devices implementing Voice Codecs, echo cancellation,
                      silence supression, etc.

  @xref
    <f OtxDrvClose>
    <nl><t OtxLogicalDeviceTypeE>
    <nl><t OtxPhysicalDeviceTypeE>
    <nl><t OTX_HANDLE>

******************************************************************************/
OTX_RESULT OtxDrvCreateLogicalDevice(
  IN  OTX_HANDLE      hHostDevice,     // @parm Parent (typically a physical device) who owns the logical device we are connecting to
  IN  OtxLogicalDeviceTypeE eDevType,  // @parm Class of device we are looking for, as known by the module in question. See also <t OtxLogicalDeviceTypeE>
  IN  OTX_HANDLE      hEventQueue,     // @parm Handle to separate event queue (0 creates device specific event queue)
  IN  OTX_INT32       nEventSourceId,  // @parm User specified ID to be used by this device as the Source ID for the events generated by the device
  OUT OTX_HANDLE      *phLogicDevice   // @parm Handle to the new logical device (if result code OTX_S_OK)
);


/******************************************************************************

  @func
    Enables a physical or logical driver device. Enabling a device makes the
    object to accept I/O requests and enables event notification if so
    configured.

  @xref
    <f OtxDrvDisable>
    <nl><f OtxDrvReset>
    <nl><f OtxDrvCancelTask>
    <nl><t OTX_HANDLE>

******************************************************************************/
OTX_RESULT OtxDrvEnable(
  IN OTX_HANDLE hObject,      // @parm Any <t OTX_HANDLE>
  IN OTX_TASK_REF nTaskRef,   // @parm See <t OTX_TASK_REF>
  IN OTX_TIME nWaitMaxMs      // @parm See <t OTX_TIME>
);


/******************************************************************************

  @func
    Disables a driver object. Disabling an object makes the object reject
    all IO requests. Only certain control commands can be issued to a disabled
    object (such as configuration and reset commands). A disabled object can not
    generate events. Events still in the event queue are not removed by this
    command.

  @xref
    <f OtxDrvEnable>
    <nl><f OtxDrvReset>
    <nl><f OtxDrvCancelTask>
    <nl><t OTX_HANDLE>

******************************************************************************/
OTX_RESULT OtxDrvDisable(
  IN OTX_HANDLE hObject,     // @parm Any <t OTX_HANDLE>
  IN OTX_TASK_REF nTaskRef,  // @parm See <t OTX_TASK_REF>
  IN OTX_TIME nWaitMaxMs     // @parm See <t OTX_TIME>
);




// @todo Not all devices maintain a public state yet
/******************************************************************************

  @func
    Gets the status of a device.

  @comm
    Any device can be in one of the 3 states:
    "WORKING," "MANUALLY BLOCKED," or "AUTO BLOCKED."  Disabling device with function
    <f OtxDrvDisable> forces the device to a state MANUALLY BLOCKED. Enabling of
    a device with function <f OtxDrvEnable> attempts to set the device into a
    WORKING state. However, if the device does not operate correctly, e.g., it
    generates excessive amounts of interrupts, the driver can automatically
    block the device and force it to state "AUTO BLOCKED."

  @xref
    <f OtxDrvEnable>
    <nl><f OtxDrvDisable>
    <nl><t OtxDeviceStateE>

******************************************************************************/
OTX_RESULT OtxDrvGetState(
  IN OTX_HANDLE        hObject,     // @parm Handle to a driver object to get state info about
  OUT OtxDeviceStateE  *peDevState  // @parm State of the device. See also <t OtxDeviceStateE>
);


/******************************************************************************

  @func
    Resets a device. It is not normally necessary to reset a device.
    A reset is normally done to resolve and error condition. Some objects must be
    disabled before they can be reset.

    Events in the event queue for the particular object are removed by this
    command.

  @xref
    <f OtxDrvEnable>
    <nl><f OtxDrvDisable>
    <nl><t OTX_HANDLE>

******************************************************************************/
OTX_RESULT OtxDrvReset(
  IN OTX_HANDLE hObject      // @parm Handle to a object to reset
);


/******************************************************************************

  @func
    Close a physical or logical device or event queue or pipe.

  @comm
    If the device is being used it may not be possible to close it.
    E.g. no devices may be connected to an event queue in order for
    the closing of the event queue to succeed.

  @xref
    <f OtxDrvOpenPhysicalDevice>
    <nl><f OtxDrvCreateLogicalDevice>
    <nl><t OTX_HANDLE>

******************************************************************************/
OTX_RESULT OtxDrvClose(
  IN OTX_HANDLE hObject     // @parm Object handle to a physical/logical device, pipe or event queue
);


/******************************************************************************

  @func
    Create a logical pipe. (One direction, one end is input, other is output)
    A pipe is always needed to carry data between two physical devices or between a
    physical device and a logical device.

  @comm
    A Pipe is a logical entity representing a connection between two or more devices.
    A Pipe has a direction, thus two Pipes are needed for a duplex connection. A pipe can
    connect any two devices regardless of where these two devices are located.
    The devices can be either physical or logical or both.

    A pipe can be created for a data rate between 8 kbit/s and 2.048Mbit/s
    (2.048 Mbit/s max is standard. Highways can be operated at up to 8.192Mbits/s).
    A pipe is a configured entity and it is overlaid on internal highways.
    A pipe can constitute a single time-slot on a Highway or it can be a superchannel
    of any number of time slots. The pipe is mapped on the physical highways when
    it is connected to a device with function <f OtxDrvConnectPipe>

  @xref
    <f OtxDrvConnectPipe>
    <nl><f OtxDrvClose>
    <nl><f OtxDrvDisconnectPipe>
    <nl><t OTX_HANDLE>

******************************************************************************/
OTX_RESULT OtxDrvCreatePipe(
  IN  OTX_INT32   nCapacity,   // @parm Capacity in kbps (only 8kbps increments allowed)
  OUT OTX_HANDLE *phPipe       // @parm Handle to the new pipe device (if result code OTX_S_OK)
);



/******************************************************************************

  @func
    Connects one end of a Pipe to a Physical Device.

  @comm
    A pipe is always used to connect two physical devices and it is always forms
    a one-to-one data path between two physical devices. I.e. every pipe must
    have exactly one physical device connected to its input and exactly one physical
    device connected to its output.

    Multiple logical devices (implemented within a physical device) can be
    connected within a physical device to use data from a pipe output. To connect
    logical devices to each other and to a pipe, use function
    <f OtxDrvConnectLogicalDevice>

    The connection mask is only needed for devices where an exact mapping
    of the pipe onto a physical interface (i.e. specific time-slots) is needed,
    e.g. in the case of a multichannel line interface where a specific time slot is
    used for signalling. It is important that the mapping matches the capacity of
    the pipe exactly. If the time-slots to be used do not matter, or if they cannot
    be changed by the user, specify coMaskSize = 0 and sqConnectMask = NULL.

  @devnote
    This function can only used for Physical devices. Logical Devices have
    to be connected with the <f OtxDrvConnectLogicalDevice> function.

  @xref
    <f OtxDrvCreatePipe>
    <nl><f OtxDrvClose>
    <nl><f OtxDrvDisconnectPipe>
    <nl><f OtxDrvOpenPhysicalDevice>
    <nl><f OtxDrvConnectLogicalDevice>
    <nl><t OtxPipeEndE>
    <nl><t OTX_HANDLE>

******************************************************************************/
OTX_RESULT OtxDrvConnectPipe(
  IN  OTX_HANDLE  hPipe,           // @parm Pipe to be connected to a Physical Device.
  IN  OtxPipeEndE ePipeEnd,        // @parm End of the pipe (input vs. output) to be connected. See Also <t OtxPipeEndE>
  IN  OTX_HANDLE  hDevice,         // @parm Physical Device to be connected to
  IN  OTX_INT32   nHighway,        // @parm Physical Highway to be used. -1 = don't care
  IN  OTX_UINT8   sqConnectMask[], // @parm Connection mask for mapping the Pipe Bits to time-slots and bits
                                   //       on the PCM highway connected to the physical device
                                   //       Can be NULL if don't care and coMaskSize = 0
                                   //       Each octet in the mask represents a timeslot on the highway
  IN  OTX_UINT32   coMaskSize      // @parm Size of connection mask for the highway connection (in octets). 0 = don't care.
);


/******************************************************************************

  @func
    Disconnects one end of a Pipe from a Physical Device.

  @comm
    After diconnection, the pipe can be re-connected to another physical
    device with function <f OtxDrvConnectPipe>

  @xref
    <f OtxDrvConnectPipe>
    <nl><f OtxDrvCreatePipe>
    <nl><f OtxDrvClose>
    <nl><f OtxDrvConnectLogicalDevice>
    <nl><t OtxPipeEndE>
    <nl><t OTX_HANDLE>

******************************************************************************/
OTX_RESULT OtxDrvDisconnectPipe(
  IN OTX_HANDLE  hPipe,             // @parm Pipe to be disconnected
  IN OtxPipeEndE ePipeEnd           // @parm End of the pipe to be disconnected. See Also <t OtxPipeEndE>
);


/******************************************************************************

    Creating a CT Bus board interconnect device to represent a physical connection
    between two or more boards.

    One entry is added for each CT Bus board interconnect in use, i.e. one for each cable.
    Then an entry is added for each board, indicating which CT Bus board interconnect
    is connecting to that board.

    Here is an example:


REGEDIT4

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\OtxDrv\Parameters]

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\OtxDrv\Parameters\Board]

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\OtxDrv\Parameters\Board\T2PX06990036]
"BoardInterConnect"=dword:00000001

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\OtxDrv\Parameters\BoardInterConnect]

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\OtxDrv\Parameters\BoardInterConnect\0]
"HwyTimeSlots"=dword:00000020

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\OtxDrv\Parameters\BoardInterConnect\1]
"HwyTimeSlots"=dword:00000020

[HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\OtxDrv\Parameters\T2PX06990036]
"BoardInterConnect"=dword:00000000


    Note that the HwyTimeSlots must be present to signify the presence of a CT Bus board
    interconnect cable. The number determines the speed of the interconnect in
    timeslots per highway. For now only the value 32 is allowed (20 hex).


******************************************************************************/

/******************************************************************************

  @func
    Inform the driver of the physical board connections for a CT Bus. This function
    should be called once for every board which is connected to this CT Bus.

  @comm
    A CT Bus device is a physical device representing the CT Bus cable
    connection between two or more boards.

    When two ore more boards are connected together with the appropriate CT Bus 
    board interconnect cable it is possible to connect two devices with a pipe
    even if they are located on different board. Before pipes can connect over 
    separate boards the driver must be informed about what types of board 
    interconnects exists, and how the boards are connected. This is done by calling
    this function.

    The CT Bus device must be opened by calling <f OtxDrvOpenPhysicalDevice> prior to 
    calling this function. The same applies to all the boards which are connected 
    to the CT Bus.

    The CT Bus does not have a direction like a pipe. The direction of each individual
    timeslot is determined at the time a Pipe Device must utilize the CT Bus
    to reach a different board. The direction of the pipe then determines the
    direction of that timeslot in the CT Bus.

    The OTX driver supports a CT bus capacity of 2048 kbps which corresponds 
    to 32 timeslots/highway.

  @xref
    <f OtxDrvOpenPhysicalDevice>
    <e OtxPhysicalDeviceTypeE.OTX_DEVICE_CT_BUS>

******************************************************************************/
OTX_RESULT OtxDrvConnectBoard(
  IN OTX_HANDLE hCtBus,               // @parm Handle to a CT Bus 
  IN OTX_HANDLE hBoard                // @parm Handle to a board (Nic) device which is connected to this CT bus
);


/******************************************************************************

  @func
    Connect an input of a Logical device to another logical device or to a pipe
    connected to the physical device implementing the logical device.

  @comm
    Two logical devices (which are implemented within one physical device)
    or a logical device and a pipe connected to the physical device implementing
    the logical device can be connected in series with this function. If the devices
    need to be connected in parallel, call this function twice with the same
    output device but with two different input devices.

    The connection point index is only needed for devices that has more than
    one inputs of outputs, e.g. a mixer or a conference bridge, and even in that
    case it can be left as -1 if the connection points are 100% symmetrical,
    e.g. a symmetrically mixed conference bridge.

  @devnote
    This function can only be used for logical devices and pipes. Physical
    devices must be connected using the function <f OtxDrvConnectPipe>

  @xref
    <f OtxDrvDisconnectLogicalDevice>
    <nl><f OtxDrvConnectPipe>
    <nl><f OtxDrvCreateLogicalDevice>
    <nl><t OTX_HANDLE>

******************************************************************************/
OTX_RESULT OtxDrvConnectLogicalDevice(
  IN OTX_HANDLE hFromDevice,       // @parm Logical Device or Pipe whose output is to be connected
  IN OTX_INT32  iFromConnectPoint, // @parm Output connection point to connect, -1 = don't care
  IN OTX_HANDLE hToDevice,         // @parm Logical Device or Pipe whose input is to be connected
  IN OTX_INT32  iToConnectPoint    // @parm Connection point on the second device to connect to, -1 = don't care
);


/******************************************************************************

  @func
    Connect an input of a Logical device to another logical device or to a pipe
    connected to the physical device implementing the logical device.

  @comm
    Two logical devices (which are implemented within one physical device)
    or a logical device and a pipe connected to the physical device implementing
    the logical device can be connected in series with this function. If the devices
    need to be connected in parallel, call this function twice with the same
    output device but with two different input devices.

    The connection point index is only needed for devices that has more than
    one inputs of outputs, e.g. a mixer or a conference bridge, and even in that
    case it can be left as -1 if the connection points are 100% symmetrical,
    e.g. a symmetrically mixed conference bridge.

  @devnote
    This function can only be used for logical devices and pipes. Physical
    devices must be connected using the function <f OtxDrvConnectPipe>

  @xref
    <f OtxDrvConnectLogicalDevice>
    <nl><f OtxDrvConnectPipe>
    <nl><f OtxDrvCreateLogicalDevice>
    <nl><t OTX_HANDLE>

******************************************************************************/
OTX_RESULT OtxDrvDisconnectLogicalDevice(
  IN OTX_HANDLE hFromDevice,       // @parm Logical Device or Pipe whose output is to be disconnected
  IN OTX_INT32  iFromConnectPoint, // @parm Output connection point to be disconnected, -1 = disconnect all
  IN OTX_HANDLE hToDevice,         // @parm Logical Device or Pipe whose input is to be disconnected
  IN OTX_INT32  iToConnectPoint    // @parm Connection point on the second device to be disconnect to, -1 = disconnect all
);


/******************************************************************************

  @func
    Create a Notification Event

  @comm
    An Event Handle if registered with an Event Queue and can be used by
    a Win32 app to wait efficiently on events to be added to the event
    queue. That way unnecessary polling with the <f OtxDrvGetEventData> can be
    avoided.

  @devnote
    Only implemented for Win32 platforms

  @xref
    <f OtxDrvWaitForSingleEvent>
    <nl><f OtxDrvWaitForMultipleEvents>
    <nl><f OtxDrvCreateEventQueue>
    <nl><f OtxDrvCloseEvent>
    <nl><t OTX_EVENT>

******************************************************************************/
OTX_RESULT OtxDrvCreateEvent(
  OUT OTX_EVENT *phEvent    // @parm Event handle, ready for use. See Also <t OTX_EVENT>
);


/******************************************************************************

  @func
    Set an event into a signaled state.

  @comm
    This function allows a previously created event to be set into signalled
    state. This function can be used by layered applications where lower layer
    module can signal an event to upper layer module which can be waiting
    efficiently using the <f OtxDrvWaitForSingleEvent> function.

  @devnote
    Only implemented for Win32 platforms

  @xref
    <nl><f OtxDrvCreateEvent>
    <nl><f OtxDrvWaitForSingleEvent>
    <nl><f OtxDrvWaitForMultipleEvents>
    <nl><f OtxDrvCloseEvent>
    <nl><t OTX_EVENT>

******************************************************************************/
OTX_RESULT OtxDrvSignalEvent(
  IN OTX_EVENT hEvent    // @parm Handle to an Event to be set into signalled state. See Also <t OTX_EVENT>
);



/******************************************************************************

  @func
    Close a Notification Event

  @comm
    Make sure there are no event queues that use event. Closing an event
    that is still in use will crash the system eventually.

  @devnote
    Only implemented for Win32 platforms

  @xref
    <f OtxDrvWaitForSingleEvent>
    <nl><f OtxDrvWaitForMultipleEvents>
    <nl><f OtxDrvCreateEventQueue>
    <nl><f OtxDrvCloseEvent>
    <nl><t OTX_EVENT>

******************************************************************************/
OTX_RESULT OtxDrvCloseEvent(
  IN OTX_EVENT hEvent    // @parm Event handle to close. See Also <t OTX_EVENT>
);


/******************************************************************************

  @func
    Create a Notification Event Queue

  @comm
    An Event Queue is used to serialize asynchronous events detected by the
    driver. Events can be anything that occur spontaneously or as a result
    of a user incurred action. It's the users responsibility to pump event
    messages out of the queue and act on them accordingly. Some messages
    are complete in the sense that they contain all infomation associated
    with the event, whereas other actions require the user to read out more
    info from the object itself (the object handle is always passed in the
    event message).

  @xref
    <f OtxDrvCreateEvent>
    <nl><f OtxDrvOpenPhysicalDevice>
    <nl><f OtxDrvCreateLogicalDevice>
    <nl><t OTX_EVENT>
    <nl><t OTX_HANDLE>

******************************************************************************/
OTX_RESULT OtxDrvCreateEventQueue(
  IN  OTX_UINT32 nSize,              // @parm Size of the queue to be created in # of messages
  IN  OTX_EVENT  hEvent,             // @parm Event which a Win32 app can
                                     // Wait for events to occur, NULL if
                                     // no notification needed (or if not a Win32 application).
  OUT OTX_HANDLE *phEventQueue       // @parm Event Queue handle

);



/******************************************************************************

  @func
    Fetch information about events that occurred in the driver.

  @comm
    Data about the events are returned in the order they occured.

    Each event is time stamped.

    Win32 applications can wait efficiently for events to occur by setting
    up a Win32 Event with the driver, see <f OtxWaitForSingleEvent>

  @xref
    <f OtxDrvCreateEventQueue>
    <nl><f OtxDrvGetEventDataEx>
    <nl><f OtxDrvEnable>
    <nl><f OtxDrvDisable>
    <nl><f OtxDrvReset>
    <nl><t OtxEventDataS>

******************************************************************************/
OTX_RESULT OtxDrvGetEventData(
  IN  OTX_HANDLE hEventQueue,         // @parm Event queue to read event from
  OUT OtxEventDataS *pEventData       // @parm Pointer to place-holder for event data
                                      //       See also <t OtxEventDataS>
);


/******************************************************************************

  @func
    Fetch multiple events that occurred in the driver. This function has the same
    functionality as the <f OtxDrvGetEventData> function, except that this function
    is able to fetch multiple event from the driver in one call (and thus reduce 
    overhead introduce by context-switching).

  @xref
    <f OtxDrvGetEventData>    
    <nl><t OtxEventDataS>

******************************************************************************/
OTX_RESULT OtxDrvGetEventDataEx(
  IN  OTX_HANDLE hEventQueue,         // @parm Event queue to read event from
  OUT OtxEventDataS *psqEventData,    // @parm Pointer to place-holder array for event data (<t OtxEventDataS>)
  IN OTX_INT32 coEventData,           // @parm Capacity of <p psqEventData> (in number of <t OtxEventDataS> elements), i.e. maximum number
                                      // of events which the driver can entry in <p psqEventData>.
  OUT OTX_INT32 *pcoEventsReturned    // @parm Number of events returned by this function (number of elements which the driver entered in <p psqEventData>).
);


/******************************************************************************

  @func
    Place an event data to an event queue.

  @comm
    This function allows a module to place data into a previously created
    event queue. This function can be used by layered applications where lower
    layer module can pass an event to an upper layer module which can be
    waiting efficiently using the <f OtxDrvWaitForSingleEvent> function.

    The driver will time stamp the event.

  @xref
    <f OtxDrvCreateEventQueue>
    <nl><f OtxDrvGetEventData>
    <nl><t OtxEventDataS>

******************************************************************************/
OTX_RESULT OtxDrvSetEventData(
  IN  OTX_HANDLE       hEventQueue,     // @parm Event queue to be used
  IN  OTX_UINT32       eDeviceType,     // @parm Device Type with which the event is associated
  IN  OTX_UINT32       nSourceId,       // @parm Device Id of the device that the event is associated with (supplied by the user when the device for opened/created)
  IN  OTX_HANDLE       hDevice,         // @parm Handle to the Device with which the event is associated
  IN  OTX_NOTIFY_CAUSE nCode,           // @parm Device specific cause code indicating what happened. Each device types has it own set of event cause codes
  IN  OTX_TASK_REF     nRequestId,      // @parm If the message is for a previous user request this member holds the user assigned request ID
  IN  OTX_UINT32       nParam           // @parm Optional parameter with user defined meaning
);


/******************************************************************************

  @func
    Wait for a notification event from the driver

  @comm
    Waits efficiently using a Win32 event.

    Specify nMilliseconds = <t OTX_TIME_INFINITY> to wait forever.

    @rdesc Returns one of the following values:

    @flag OTX_E_HANDLE | Handle invalid
    @flag OTX_S_TIMEOUT | Wait timed out
    @flag OTX_S_OK | Event occured
    @flag OTX_E_FAIL | Non-specified failure

  @xref
    <f OtxDrvWaitForMultipleEvents>
    <nl><f OtxDrvCreateEvent>
    <nl><t OTX_EVENT>

******************************************************************************/
OTX_RESULT OtxDrvWaitForSingleEvent(
  IN OTX_EVENT hEvent,     // @parm Event handle, initilized with <f OtxDrvCreateEvent>
  IN OTX_TIME  nWaitMaxMs  // @parm Max time to wait in millisconds
);


/******************************************************************************

  @func
    Wait for a notification event from the driver

  @comm
    Waits efficiently using a Win32 event.

    Specify nMilliseconds = OTX_TIME_INFINITY to wait forever.

  @xref
    <f OtxDrvWaitForSingleEvent>
    <nl><f OtxDrvCreateEvent>
    <nl><t OTX_EVENT>
    <nl><t OTX_BOOL>

******************************************************************************/
OTX_RESULT OtxDrvWaitForMultipleEvents(
  IN  OTX_INT32  coEventHandles,  // @parm Event handle count
  IN  OTX_EVENT  sqhEvents[],     // @parm Array of event handles. See Also <t OTX_EVENT>
  IN  OTX_TIME   nWaitMaxMs,      // @parm Max time to wait in millisconds
  IN  OTX_BOOL   bWaitAll,        // @parm Specifies the wait type. If OTX_TRUE, the function returns
                                  //       when the state all objects in the lpHandles array is signaled.
                                  //       If OTX_FALSE, the function returns when the state of any
                                  //       one of the objects set to is signaled. In the latter case,
                                  //       the return value indicates the object whose state caused the
                                  //       function to return.  See also <t OTX_BOOL>
  OUT OTX_INT32 *pnEventFired     // @parm index to event in array that fired
);




/******************************************************************************

  @func
    Cancels a pending task for a particular device.

  @comm
    If a device has several tasks with the same reference number the oldest
    task is cancelled. Only one or zero tasks are removed for each call.

  @xref
    <t OTX_TASK_REF>

******************************************************************************/
OTX_RESULT OtxDrvCancelTask(
  IN OTX_HANDLE hDevice,    // @parm Handle to the device
  IN OTX_TASK_REF nTaskRef  // @parm Callers task reference, as given earlier. See <t OTX_TASK_REF>
);


/******************************************************************************

  @func
    Transparent device communication.

  @comm
    Mostly used to communicate
    with firmware modules and firmware devices. Since the capabilities
    of the firmware are transparent to the driver this is the only way
    firmware modules and devices can be controlled by an application
    using the OtxDrv API.
  @xref
    <t OTX_HANDLE>
    <nl><t OTX_TASK_REF>
    <nl><t OTX_TIME>

******************************************************************************/
OTX_RESULT OtxDrvDeviceControl(
  IN  OTX_HANDLE   hDevice,        // @parm Handle to device
  IN  OTX_INT32    nCommandCode,   // @parm Code indicating requested operation
  IN  OTX_UINT8    *pInBuffer,     // @parm Input parameters in serial format
  IN  OTX_UINT32   cbInBuffer,     // @parm Byte count of input parameters
  OUT OTX_UINT8    *pOutBuffer,    // @parm Buffer to receive output parameters in serial format
  IN  OTX_UINT32   cbOutBuffer,    // @parm Size in bytes of output buffer. Must be large enough for the given command
  OUT OTX_UINT32   cbReturned,     // @parm Actual number of bytes returned (may differ from cbOutBuffer).
  IN  OTX_TASK_REF nTaskRef,       // @parm See <t OTX_TASK_REF>
  IN  OTX_TIME     nWaitMaxMs      // @parm See <t OTX_TIME>
);


/******************************************************************************

  @func
    Polls for Events from the driver.

  @comm
    This function can be used if interrupts can not be used for some reason.
    Instead of an interrupt service routine servicing interrupts upon arrival,
    the application can constantly poll this function and force the interrupt
    service routine to be executed periodically without having interrupts
    enabled.

******************************************************************************/
OTX_RESULT OtxDrvPollEvents(
  IN  OTX_HANDLE hDevice      // @parm Handle to device
);

/******************************************************************************
  @func
    Initializes an iterator so that it can be used to retrieve data for the
    first element/device in a sequence

  @xref
    <f OtxDrvEnumDeviceType>
    <nl><f OtxDrvEnumDevice>
    <nl><f OtxDrvIsIteratorEnd>
    <nl><t OTX_ITERATOR>


******************************************************************************/
OTX_RESULT OtxDrvIteratorBegin(OTX_ITERATOR *pItr);

/******************************************************************************
  @func
    Checks an iterator to see if it has reached past the end of the sequence
        being enumerated.

  @xref
    <f OtxDrvIteratorBegin>
    <nl><f OtxDrvIteratorNext>
    <nl><t OTX_ITERATOR>


******************************************************************************/
OTX_RESULT OtxDrvIsIteratorEnd(OTX_ITERATOR *pItr);

/******************************************************************************
  @func
    Enumerates over all device types supported by a parent device

  @comm
    Retrievs data for item referenced by the iterator

    The iterator is incremented to reference the next item in the
    sequence. It can be used in a subsequent call to the same function
    to fetch data for the next element in the same sequence. Repeated calls
    to the same function with the same iterator eventually fetches data for
    all items in this sequence.

    The iterator must already be initialized with a call to <f OtxDrvIteratorBegin>
    before the first call to this function.

    It is important that the iterator is used on the same sequence of items between
    each call to <f OtxDrvIteratorBegin>, i.e. the same function must be called with
    the same handle and the same iterator.

    However, it is possible for multiple iterators to be used on the same sequence
    independently, at the same time, even in separate threads.

  @xref
    <f OtxDrvEnumDevice>
    <nl><f OtxDrvIteratorBegin>
    <nl><f OtxDrvIsIteratorEnd>
    <nl><t OTX_ITERATOR>

******************************************************************************/
OTX_RESULT OtxDrvEnumDeviceType(
  IN     OTX_HANDLE   hDevice,            // @parm Handle to the physical parent device
  IN OUT OTX_ITERATOR *pItr,              // @parm Enumeration start position. See <t OTX_ITERATOR>
  OUT    OTX_INT32    *peDeviceType       // @parm Physical or logical device type. See <t OtxPhysicalDeviceTypeE> and <t OtxLogicalDeviceTypeE>
);


/******************************************************************************
  @func
    Enumerates over all device numbers for a specific type of devices supported
    by a parent device.

  @comm
    Retrievs data for item referenced by the iterator

    After the calll the iterator has been incremented to reference the next item in the
    sequence. It can be used in a subsequent call to the same function
    to fetch data for the next element in the same sequence. Repeated calls
    to the same function with the same iterator eventually fetches data for
    all items in this sequence. Note that it is not possible to increment the iterator in
    any other way.

    The iterator must already be initialized with a call to <f OtxDrvIteratorBegin>
    before the first call to this function.

    It is important that the iterator is used on the same sequence of items between
    each call to <f OtxDrvIteratorBegin>, i.e. the same function must be called with
    the same handle value (hDevice) and the same iterator (pItr).

    However, it is possible for multiple iterators to be used on the same sequence
    independently, at the same time, even in separate threads.

  @xref
    <f OtxDrvEnumDeviceType>
    <nl><t OtxDrvIteratorBegin>
    <nl><t OtxDrvIsIteratorEnd>
    <nl><t OTX_ITERATOR>


******************************************************************************/
OTX_RESULT OtxDrvEnumDevice(
  IN     OTX_HANDLE   hDevice,            // @parm Handle to the physical parent device
  IN     OTX_INT32    eDeviceType,        // @parm Physical or logical device type. See <t OtxPhysicalDeviceTypeE> and <t OtxLogicalDeviceTypeE>
  IN OUT OTX_ITERATOR *pItr,              // @parm Enumeration start position. See <t OTX_ITERATOR>
  OUT    OTX_HANDLE   *phChildDevice      // @parm Handle to device referenced by iterator. This handle can be always be used to query the device, but the device must have been opened for most other operations to succeed
);


/******************************************************************************
  @func
    Translate an Event message into english text describing the event.

  @comm
    The event message string is copied into a user supplied buffer
******************************************************************************/
OTX_RESULT OtxDrvEventCode2String(
  IN  OTX_HANDLE hDevice,        // @parm Handle to the device who generated the event
  IN  OTX_INT32  nEventCode,     // @parm Event Cause Code to be translated. See also <t OtxEventDataS>
  IN  OTX_UINT32 nBufLen,        // @parm Length of the User supplied String Buffer
  OUT char       szStrBuf[]      // @parm User supplied String buffer
);


/******************************************************************************
  @func
    Translate a result code into english text describing the code.

  @comm
    The result message string is copied into a user supplied buffer
******************************************************************************/
OTX_RESULT OtxDrvResultCode2String(
  IN  OTX_HANDLE hDevice,        // @parm Handle to the device who generated the result code
  IN  OTX_RESULT nResultCode,    // @parm Result Code to be translated. See also <t OTX_RESULT>
  IN  OTX_UINT32 nBufLen,        // @parm Length of the User supplied String Buffer
  OUT char       szStrBuf[]      // @parm User supplied String buffer
);


/******************************************************************************
    @func
    Read a 16-bit register from a physical device
******************************************************************************/
OTX_RESULT OtxDrvReadReg16(
  IN  OTX_HANDLE hDevice,        // @parm Handle to the device to read from
  IN  OTX_UINT32 nAddress,       // @parm Register address within the device
  OUT OTX_UINT32 *pnValue        // @parm 16-bit value read from the device
);

/******************************************************************************
    @func
    Read an 8-bit register from a physical device
******************************************************************************/
OTX_RESULT OtxDrvReadReg8(
  IN  OTX_HANDLE hDevice,        // @parm Handle to the device to read from
  IN  OTX_UINT32 nAddress,       // @parm Register address within the device
  OUT OTX_UINT32 *pnValue        // @parm 8-bit value read from the device
);

/******************************************************************************
    @func
    Write a 16-bit value to a 16-bit register in a physical device
******************************************************************************/
OTX_RESULT OtxDrvWriteReg16(
  IN  OTX_HANDLE hDevice,        // @parm Handle to the device to write to
  IN  OTX_UINT32 nAddress,       // @parm Register address within the device
  OUT OTX_UINT32 nValue          // @parm 16-bit value to be written to the device
);

/******************************************************************************
    @func
    Write an 8-bits value to an 8-bit register in a physical device
******************************************************************************/
OTX_RESULT OtxDrvWriteReg8(
  IN  OTX_HANDLE hDevice,        // @parm Handle to the device to write to
  IN  OTX_UINT32 nAddress,       // @parm Register address within the device
  OUT OTX_UINT32 nValue          // @parm 8-bit value to be written to the device
);

/******************************************************************************
  // Debug function
******************************************************************************/
OTX_RESULT OtxDrvDbgDump(
  IN  OTX_HANDLE hDevice,
  IN  OTX_UINT32 nCmd
);


#ifdef __cplusplus
}  // extern "C"
#endif

/******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 1998, 1999


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

******************************************************************************/
#endif  // OTX_DRV_H
