/* $Workfile: OtxJtagD.h $   $Revision: 1 $    $Modtime: 6/21/01 2:22p $  */
#ifndef OTX_HDLC_DEF_H
#define OTX_HDLC_DEF_H
/*******************************************************************************

  @doc PublicHwApi OtxHdlcD

  Copyright (C) 1998, 1999 Odin TeleSystems Inc., All Rights Reserved

  OTX Generic Hardware Driver API

  @module
  OtxHdlcD | Constants, Macros, and Types for the HDLC Functions

  @contents2
  OtxHdlcD.h |

  @index enum type struct | OtxHdlcD

  @contents2
  OTX Hdlc Device Macros, Constants, and Types |

  @head3 OtxHdlcXXX Macro, Constant, and Type Reference |

  @index enum type struct | OtxHdlcD

*******************************************************************************/

#include "OtxType.h"
#include "OtxErr.h"

#ifdef __cplusplus
  extern "C" {
#endif

#ifdef __linux__
#pragma pack(1)
#else
#pragma pack(push,1)
#endif


/*---------------------------------------------------------------------------*\
  @enum
    Control commands to the TBC device.
    
  @xref
    <f OtxJtagControl>

\*---------------------------------------------------------------------------*/
enum OtxJtagControlCmdE {
  OTX_JTAG_CLK_PULSE      = 0x203, // @emem  Issue a positive pulse on TCLK                                 
  OTX_JTAG_SET_TCLK       = 0x204, // @emem  Set TCLK high                                 
  OTX_JTAG_SET_TMS        = 0x205, // @emem  Set TMS0 high; alias for OTX_JTAG_SET_TMS0   
  OTX_JTAG_SET_TDI        = 0x206, // @emem  Set TDI high                                 
  OTX_JTAG_UNSET_TCLK     = 0x207, // @emem  Set TCLK low                                 
  OTX_JTAG_UNSET_TMS      = 0x208, // @emem  Set TMS0 low; alias for OTX_JTAG_UNSET_TMS0 
  OTX_JTAG_UNSET_TDI      = 0x209, // @emem  Set TDI low.                                 
  OTX_JTAG_READ_TDO       = 0x20A, // @emem  Read bit value of TDO                                 
  OTX_JTAG_SET_TMS0       = 0x20C, // @emem  Set TMS0 high                                 
  OTX_JTAG_SET_TMS1       = 0x20D, // @emem  Set TMS1 high                                 
  OTX_JTAG_SET_TMS2       = 0x20E, // @emem  Set TMS2 high                                 
  OTX_JTAG_SET_TMS3       = 0x20F, // @emem  Set TMS3 high                                 
  OTX_JTAG_SET_TMS4       = 0x210, // @emem  Set TMS4 high                                 
  OTX_JTAG_SET_TMS5       = 0x211, // @emem  Set TMS5 high                                 
  OTX_JTAG_SET_TMS6       = 0x212, // @emem  Set TMS6 high                                 
  OTX_JTAG_SET_TMS7       = 0x213, // @emem  Set TMS7 high                                 
  OTX_JTAG_UNSET_TMS0     = 0x214, // @emem  Set TMS0 low                                 
  OTX_JTAG_UNSET_TMS1     = 0x215, // @emem  Set TMS1 low                                 
  OTX_JTAG_UNSET_TMS2     = 0x216, // @emem  Set TMS2 low                                 
  OTX_JTAG_UNSET_TMS3     = 0x217, // @emem  Set TMS3 low                                 
  OTX_JTAG_UNSET_TMS4     = 0x218, // @emem  Set TMS4 low                                 
  OTX_JTAG_UNSET_TMS5     = 0x219, // @emem  Set TMS5 low                                 
  OTX_JTAG_UNSET_TMS6     = 0x21A, // @emem  Set TMS6 low                                 
  OTX_JTAG_UNSET_TMS7     = 0x21B, // @emem  Set TMS7 low                                 
}; 

typedef enum OtxJtagControlCmdE OtxJtagControlCmdE;




/*---------------------------------------------------------------------------*\
  @enum 
    JTAG Attributes
\*---------------------------------------------------------------------------*/
enum OtxJtagAttributesE {
  OTX_ATTR_JTAG_HM2X20_HIGHZ              = OTX_ATTR_END +  0,  // @emem (OTX_ATTRIB_BOOL) Read/Write. 
                                                                // If set to OTX_TRUE the Hermod-2x20 board is tri-stated from the Hermod-PCI board.

  OTX_ATTR_JTAG_END                       = OTX_ATTR_END +  1
};

typedef enum OtxJtagAttributesE OtxJtagAttributesE;


/*---------------------------------------------------------------------------*\
  @enum
    JTAG Event Codes
\*---------------------------------------------------------------------------*/
enum OtxJtagEventCodesE {
  // None

  OTX_JTAG_END_OF_EVENT_CODES
};

typedef enum OtxJtagEventCodesE OtxJtagEventCodesE;


/*---------------------------------------------------------------------------*\
  @enum
    JTAG Error Result Codes
\*---------------------------------------------------------------------------*/
enum OtxJtagResultCodesE {
  // None 

  OTX_JTAG_RES_END = OTX_RES_END + 0
};

typedef enum OtxJtagResultCodesE OtxJtagResultCodesE;


#ifdef __linux__
#pragma pack()
#else
#pragma pack(pop)
#endif

#ifdef __cplusplus
  } // extern "C"
#endif


/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 1998, 1999


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/

#endif // OTX_HDLC_DEF_H
